import os
from typing import Tuple


def getFilePath(
    message: str,
    endswith: Tuple = (""),
    invalid_filetype_message: str = "Invalid File Type",
    folder: bool = True,
):
    while True:
        file_path = input(message)
        file_path = sanitizeFilePath(file_path)
        if not folder:
            if file_path.endswith(endswith):
                return file_path
            else:
                print(invalid_filetype_message)
        else:
            return file_path


def sanitizeFilePath(user_input: str):
    # Remove leading/trailing whitespaces and quotes from the user input
    user_input = user_input.strip(" '\"")

    # Replace backslashes with forward slashes (for cross-platform compatibility)
    user_input = user_input.replace("\\", "/")

    # Remove any leading './' from the path
    if user_input.startswith("./"):
        user_input = user_input[2:]

    # Get the absolute path to handle relative paths
    sanitized_path = os.path.abspath(user_input)

    return sanitized_path



from typing import Optional, List
from pypeepa.checks.checkInputIsInt import checkInputIsInt
from pypeepa.userInteraction.askYNQuestion import askYNQuestion


def askSelectRangeQuestion(
    question: str, min: int, max: int, null_possible: Optional[bool] = False
) -> int | List[int] | None:
    """
    Asks the user to select a range of values between min and max\n
    @param: `question`: The question to ask the user.\n
    @param: `min`: The min value\n
    @param: `max`: The max value\n
    @param: `null_posibble`: (Optional) If set to True, users can input empty string instead of the options.\n
    @return: The selected option as int, or if null_possible is set to True, then can also return None.
    """
    invalid_input = True
    while invalid_input:
        user_input = input(
            f"{question} \nTo select a range use '-'.\nTo select multiple use ','.\n eg:- 1,2,3-5,6 \n(select between {min}-{max}):"
        )
        return_value = None
        if user_input == "" and null_possible:
            invalid_input = False
        elif checkInputIsInt(user_input):
            if min <= int(user_input) <= max:
                invalid_input = False
                return_value = int(user_input)
        elif not checkInputIsInt(user_input):
            array_input = user_input.split(",")
            array_input = expandRangeElements(array_input)
            array_input = convertStrToInt(array_input)
            array_input = checkRangeInList(array_input, min, max)
            print("Selected: ", array_input)
            confirm = askYNQuestion("Confirm selection?")
            if confirm:
                return_value = array_input
                invalid_input = False
        else:
            print(f"\x1B[31mInvalid input! Please input within\x1B[37m {min}-{max}")
    return return_value


def checkRangeInList(lst, min, max):
    checked_list = []
    for x in lst:
        if min <= x <= max:
            checked_list.append(x)
        else:
            print("Found element out of range: ", x)
    return checked_list


def convertStrToInt(lst):
    converted_list = []

    for x in lst:
        try:
            if isinstance(x, str) and x.isdigit():
                converted_list.append(int(x))
        except ValueError:
            print("Value Error on:", x)
            pass

    return converted_list


def expandRangeElements(array):
    expanded_array = []

    for element in array:
        try:
            if "-" in str(element):
                start, end = map(int, element.split("-"))
                expanded_array.extend(map(str, range(start, end + 1)))
            else:
                expanded_array.append(str(element))
        except ValueError:
            print("Value Error on:", element)
            pass
    return expanded_array



import os
import pandas
from typing import Literal


def writeFile(
    data_to_write: pandas.DataFrame,
    file_path: str,
    file_name: str,
    file_type: Literal["csv", "xlsx", "json"],
):
    file = file_path + file_name + "." + file_type
    # Create output directory if it doesnt exist
    if not os.path.exists(file_path):
        os.makedirs(file_path)
        print("\nDirectory created in your documents:", file_path)
    try:
        if file_type == "xlsx":
            # writing to excel file using openpyxl
            with pandas.ExcelWriter(file, engine="openpyxl") as writer:
                data_to_write.to_excel(writer, "spreadsheet1")
        elif file_type == "csv":
            data_to_write.to_csv(file)
        elif file_type == "json":
            json_string = data_to_write.to_json(orient="records")
            with open(file, "w") as json_file:
                json_file.write(json_string)
        else:
            raise Exception("Unsupported file_type. Use 'csv', 'xlsx', or 'json'.")

        print("\nOutput file saved to:", file)

    except Exception as exc:
        raise Exception("\nFailed to save output file!", exc)



from alive_progress import alive_bar
from typing import Iterable, Any, Optional


def progressBarIterator(
    iterable: Iterable[Any],
    iterableLength: Optional[int] = None,
    description: Optional[str] = "Progress",
    **kwargs
):
    if iterableLength == None:
        iterableLength = len(iterable)
    with alive_bar(
        iterableLength, force_tty=True, bar="filling", spinner="waves", **kwargs
    ) as bar:
        bar.title = description
        for item in iterable:
            yield item
            bar()



from .askYNQuestion import askYNQuestion
from .printArray import printArray
from .signature import signature
from .getFilePath import getFilePath, sanitizeFilePath
from .progressBarIterator import progressBarIterator
from .askHeaderForMultipleCSV import askHeaderForMultipleCSV
from .askSelectOptionQuestion import askSelectOptionQuestion
from .askSelectRangeQuestion import askSelectRangeQuestion
